@@ -1,6 +1,8 @@
 from ..core.constants import MemberLevel
+from enum import Enum, unique
 
 DISTRIBUTE_ERROR_CODE_PREFIX = 246000
+BASE_DISTRIBUTE_API_CODE = 20103000
 
 
 class DistributeStatusOwner(object):
@@ -78,6 +80,8 @@ class DistributeStatus(object):
     DEPOSITPAY_RETURN = "depositpay_return"
     # 海淘初始态
     HAITAO_INIT = "haitao_init"
+    # 海淘已推百世快递
+    HAITAO_BAISHI = "haitao_baishi"
     # 海淘已退
     HAITAO_RETURN = "haitao_return"
     # 海淘已发货
@@ -86,9 +90,19 @@ class DistributeStatus(object):
     # 滞留仓
     SERVICE_STOP = "service_stop"
 
+    # 2022.08 虚拟商品
+    # 虚拟商品INIT
+    VIRTUAL_INIT = "virtual_init"
+    # 虚拟已确收
+    CUS_CONFIRM_AT_VIRTUAL = "cus_confirm_at_virtual"
+
+    # 2022.09 特需售后单自提转快递支付运费订单的发货单
+    MAKEUP_POSTAGE_PAID = "paid"
+
     NO_SPLIT_STATUS_LIST = [
         INIT,
         HGVENDING_INIT,
+        VIRTUAL_INIT,
         PRINTED,
         CUSSERVICE_PICKED_UP,
         GUIDE_PICKED_UP,
@@ -96,6 +110,7 @@ class DistributeStatus(object):
         CUS_CONFIRM,
         CUS_CONFIRM_AT_GUIDE,
         CUS_CONFIRM_AT_CUSSERVICE,
+        CUS_CONFIRM_AT_VIRTUAL,
         CUS_CONFIRM_AT_HGVENDING,
         PACKAGED,
         SERVICE_DESK,
@@ -110,10 +125,13 @@ class DistributeStatus(object):
         TAILPAY_OVERTIME,
         DEPOSITPAY_RETURN,
         HAITAO_INIT,
+        HAITAO_BAISHI,
         HAITAO_PACKAGED,
         HAITAO_RETURN,
         SERVICE_STOP,
+        MAKEUP_POSTAGE_PAID,
     ]
+    
 
     # 可以被售后集合
     CAN_AFTERSALE_STATUS_LIST = [
@@ -124,18 +142,21 @@ class DistributeStatus(object):
         CUS_CONFIRM,
         CUS_CONFIRM_AT_GUIDE,
         CUS_CONFIRM_AT_CUSSERVICE,
+        CUS_CONFIRM_AT_VIRTUAL,
         PACKAGED,
         SERVICE_DESK,
         SERVICE_STORE,
         OUTSTORAGE_WAIT_PACKAGE,
         OUTSTORAGE_PACKAGED,
         SERVICE_STOP,
+        MAKEUP_POSTAGE_PAID,
     ]
 
     ## 初始态集合
     INIT_STATUS_LIST = [
         INIT,
         HGVENDING_INIT,
+        VIRTUAL_INIT,
         PRINTED,
         OUTSTORAGE_WAIT_PACKAGE,
     ]
@@ -146,7 +167,9 @@ class DistributeStatus(object):
         PRINTED,
         GUIDE_PICKED_UP,
         SPLIT,
-        HGVENDING_WAIT_PICKUP
+        HGVENDING_WAIT_PICKUP,
+        # 补缴运费的商品可以整单退
+        MAKEUP_POSTAGE_PAID,
     ]
 
     BEFORE_cuservice_pick_up = [
@@ -185,6 +208,7 @@ class DistributeStatus(object):
         CUS_CONFIRM: "已提货",
         CUS_CONFIRM_AT_GUIDE: "已提货",
         CUS_CONFIRM_AT_CUSSERVICE: "已提货",
+        CUS_CONFIRM_AT_VIRTUAL: "已完成",  # 2022.8.23 琪琪说改成已完成，目前前端根据这个字段，展示下面的文字描述
         SPLIT: "已拆单",
         NO_PICKUP_RETURN: "售后",
         HGVENDING_WAIT_PICKUP: "待提",
@@ -194,9 +218,11 @@ class DistributeStatus(object):
         DEPOSITPAY_RETURN: "定金已退",
         TAILPAY_OVERTIME: "尾款超时",
         HAITAO_INIT: "备货中",
+        HAITAO_BAISHI: "备货中",
         HAITAO_PACKAGED: "已发货",
         HAITAO_RETURN: "售后",
         SERVICE_STOP: "待发货",
+        MAKEUP_POSTAGE_PAID: "运费已支付",
     }
     # For B端 查询
     DISTRIBUTE_STATUS_CHOICES_FOR_B_QUERY = {
@@ -214,19 +240,22 @@ class DistributeStatus(object):
         CUS_CONFIRM: "已自提",
         CUS_CONFIRM_AT_GUIDE: "导购端已自提",
         CUS_CONFIRM_AT_CUSSERVICE: "客服端已自提",
+        CUS_CONFIRM_AT_VIRTUAL: "虚拟商品已提货",
         # SPLIT: "拆单",
         NO_PICKUP_RETURN: "未提已退",
         OUTSTORAGE_RETURN: "外仓已退",
         HGVENDING_WAIT_PICKUP: "贩售机待提",
         CUS_CONFIRM_AT_HGVENDING: "贩售机端已提货",
         HGVENDING_NO_PICKUP_RETURN: "贩售机已退",
-        DEPOSITPAY: "定金已支付",
+        DEPOSITPAY: "定金已付",
         DEPOSITPAY_RETURN: "定金已退",
-        TAILPAY_OVERTIME: "尾款支付超时",
+        TAILPAY_OVERTIME: "尾款超时",
         HAITAO_INIT: "海淘备货中",
+        HAITAO_BAISHI: "海淘已推百世",
         HAITAO_PACKAGED: "海淘已发货",
         HAITAO_RETURN: "海淘已退",
-        SERVICE_STOP: "滞留仓已收"
+        SERVICE_STOP: "滞留仓已收",
+        MAKEUP_POSTAGE_PAID: "运费已支付",
     }
 
     DISTRIBUTE_MESSAGE_DICT_FOR_C = {
@@ -241,17 +270,20 @@ class DistributeStatus(object):
         CUS_CONFIRM: "商品已提走，期待您下次光临。",
         CUS_CONFIRM_AT_GUIDE: "商品已提走，期待您下次光临。",
         CUS_CONFIRM_AT_CUSSERVICE: "商品已提走，期待您下次光临。",
+        CUS_CONFIRM_AT_VIRTUAL: "虚拟商品已提货，期待您下次光临。",
         CUSSERVICE_HOLD: "商品已售后。",
         NO_PICKUP_RETURN: "商品已售后。",
         OUTSTORAGE_RETURN: "外仓已售后。",
         HGVENDING_NO_PICKUP_RETURN: "订单超时30分钟自动退单，如有需要请再扫码下单。",
         DEPOSITPAY: "定金已支付。",
-        DEPOSITPAY_RETURN: "定金已退。",
-        TAILPAY_OVERTIME: "尾款支付超时。",
+        DEPOSITPAY_RETURN: "您的商品已退还定金",
+        TAILPAY_OVERTIME: "未按时支付尾款，您的订单已关闭",
         HAITAO_INIT: "海淘备货中。",
+        HAITAO_BAISHI: "海淘备货中。",
         HAITAO_PACKAGED: "海淘已发货。",
         HAITAO_RETURN: "海淘已售后。",
         SERVICE_STOP: "暂时无法发货，商品滞留。",
+        MAKEUP_POSTAGE_PAID: "订单自提转快递运费已支付。",
     }
 
     DISTRIBUTE_STATUS_CHOICES = (
@@ -275,17 +307,22 @@ class DistributeStatus(object):
         (HGVENDING_WAIT_PICKUP, "贩售机待提"),
         (CUS_CONFIRM_AT_HGVENDING, "贩售机端已提货"),
         (HGVENDING_NO_PICKUP_RETURN, "贩售机端已退"),
-        (DEPOSITPAY, "定金已支付"),
+        (DEPOSITPAY, "定金已付"),
         (DEPOSITPAY_RETURN, "定金已退"),
-        (TAILPAY_OVERTIME, "尾款支付超时"),
+        (TAILPAY_OVERTIME, "尾款超时"),
         (HAITAO_INIT, "海淘备货中"),
+        (HAITAO_BAISHI, "海淘已推百世"),
         (HAITAO_PACKAGED, "海淘已发货"),
         (HAITAO_RETURN, "海淘已退"),
         (SERVICE_STOP, "滞留仓已收"),
+        (VIRTUAL_INIT, "虚拟商品初始态"),
+        (CUS_CONFIRM_AT_VIRTUAL, "虚拟商品已提货"),
+        (MAKEUP_POSTAGE_PAID, "运费已支付"),
     )
 
     DISTRIBUTE_STATUS_OWNER_DICT = {
         INIT: DistributeStatusOwner.GUIDE,
+        VIRTUAL_INIT: DistributeStatusOwner.CUSTOMER,
         HGVENDING_INIT: DistributeStatusOwner.HGVENDING,
         PRINTED: DistributeStatusOwner.GUIDE,
         GUIDE_PICKED_UP: DistributeStatusOwner.GUIDE,
@@ -298,6 +335,7 @@ class DistributeStatus(object):
         CUS_CONFIRM: DistributeStatusOwner.CUSTOMER,
         CUS_CONFIRM_AT_GUIDE: DistributeStatusOwner.CUSTOMER,
         CUS_CONFIRM_AT_CUSSERVICE: DistributeStatusOwner.CUSTOMER,
+        CUS_CONFIRM_AT_VIRTUAL: DistributeStatusOwner.CUSTOMER,
         CUS_CONFIRM_AT_HGVENDING: DistributeStatusOwner.CUSTOMER,
         PACKAGED: DistributeStatusOwner.CUSTOMER,
         OUTSTORAGE_WAIT_PACKAGE: DistributeStatusOwner.OUTSTORAGE,  # 外仓
@@ -308,9 +346,11 @@ class DistributeStatus(object):
         DEPOSITPAY_RETURN: DistributeStatusOwner.GUIDE,
         TAILPAY_OVERTIME: DistributeStatusOwner.GUIDE,
         HAITAO_INIT: DistributeStatusOwner.HAITAO,
+        HAITAO_BAISHI: DistributeStatusOwner.HAITAO,
         HAITAO_PACKAGED: DistributeStatusOwner.HAITAO,
         HAITAO_RETURN: DistributeStatusOwner.HAITAO,
         SERVICE_STOP: DistributeStatusOwner.CUSSERVICE,
+        MAKEUP_POSTAGE_PAID: DistributeStatusOwner.CUSTOMER,
     }
 
     DISTRIBUTE_STATUS_DIRECTION_DICT = {
@@ -338,9 +378,12 @@ class DistributeStatus(object):
         TAILPAY_OVERTIME: DistributeDirection.GUIDE,
 
         HAITAO_INIT: DistributeDirection.HAITAO,
+        HAITAO_BAISHI: DistributeDirection.HAITAO,
         HAITAO_PACKAGED: DistributeDirection.HAITAO,
         HAITAO_RETURN: DistributeDirection.HAITAO,
         SERVICE_STOP: DistributeDirection.CUSSERVICE,
+
+        MAKEUP_POSTAGE_PAID: DistributeDirection.CUSSERVICE,
     }
 
     # 终结状态列表
@@ -350,6 +393,7 @@ class DistributeStatus(object):
         CUS_CONFIRM,
         CUS_CONFIRM_AT_GUIDE,
         CUS_CONFIRM_AT_CUSSERVICE,
+        CUS_CONFIRM_AT_VIRTUAL,
         CUS_CONFIRM_AT_HGVENDING,
         SPLIT,
         NO_PICKUP_RETURN,
@@ -362,7 +406,8 @@ class DistributeStatus(object):
         TAILPAY_OVERTIME,
         # 海淘
         HAITAO_PACKAGED,
-        HAITAO_RETURN
+        HAITAO_RETURN,
+        MAKEUP_POSTAGE_PAID,
     ]
 
     # 非最终状态
@@ -395,6 +440,16 @@ class DistributeStatus(object):
         CUS_CONFIRM_AT_CUSSERVICE,
     ]
 
+    # 未售后的不支持修改配送方式的发货单状态
+    CAN_NOT_MODY_SHIPPING_METHOD_DISTRIBUTE_STATUS_LIST = [
+        CUS_CONFIRM_AT_GUIDE,
+        CUS_CONFIRM_AT_CUSSERVICE,
+        PACKAGED,
+        OUTSTORAGE_PACKAGED,
+        OUTSTORAGE_WAIT_PACKAGE,
+        SPLIT,
+    ]
+
 
 class ShippingMethod(object):
     # 普通快递
@@ -422,6 +477,7 @@ class DistributeSyncType(object):
     VISITOR = "VISITOR"
 
 
+# 发货单操作事件名称常量
 class DistributeEventConst(object):
     NEW_ADD = "new_add"
     PRINT = "print"
@@ -457,15 +513,16 @@ class DistributeEventConst(object):
     CANCEL_EXPRESS = "cancel_express"
     MODIFY_SHIPPING_NUMBER = "modfiy_shipping_number"
     MERGE_EXPRESS = "merge_express"
+    MERGE_EXPRESS_MAIN = "merge_express_main"
     # 客服退还货物给专柜事件
     CUSSERVICE_RETURN = "cusservice_return"
     OUTSTORAGE_RETURN = "outstorage_return"
-    Add_SHIPPING_ADDRESS = "add_shipping_address"
+    ADD_SHIPPING_ADDRESS = "add_shipping_address"
     UPDATE_SHIPPING_METHOD = "update_shipping_method"
+    UPDATE_SHIPPING_METHOD_FOR_MERGE = "update_shipping_method_for_merge"
     ADD_GUIDE_NOTE = "add_guide_note"
     BLOCK_SUCC = "block_succ"
     BLOCK_FAILED = "block_failed"
-    ERROR_MESSAGE = "error_message"
     MODIFY_SHIPPING_NUMBER_FAILED = "modify_shipping_number_failed"
 
     # 货架管理 出库/入库/更改货位/合并货位
@@ -488,14 +545,13 @@ class DistributeEventConst(object):
 
     # 海淘推数据给百世
     HAITAO_PUSHDATA_2_BAISHI = "haitao_pushdata_2_baishi"
+    # 海淘取消推百世数据
+    HAITAO_CANCEL_FROM_BAISHI = "haitao_cancel_from_baishi"
     # 报错日志
     ERROR_MESSAGE = "error_message"
     # 普通日志
     NORMAL_MESSAGE = "normal_message"
 
-    # 合单催配送提醒
-    PUSH_GUIDE_ASSIGN = "push_guide_assign"
-
     # 付尾款
     PAY_TAIL = "pay_tail"
     # 退定金
@@ -507,6 +563,16 @@ class DistributeEventConst(object):
     # 合单催配送提醒
     PUSH_GUIDE_ASSIGN = "push_guide_assign"
 
+    # 2022.08 特殊售后单需求
+    AFSPLUS_OPEN = "afsplus_open"
+    AFSPLUS_CLOSE = "afsplus_close"
+    AFSPLUS_COMPLETE = "afsplus_complete"
+    AFSPLUS_S2E = "afsplus_s2e"  # 自提转快递
+    AFSPLUS_E2S = "afsplus_e2s"  # 快递转自提
+
+    # 2022.08 虚拟商品确收
+    CUS_CONFIRM_AT_VIRTUAL = "cus_confirm_at_virtual"
+
     FROM_TO_MAP = {
         PRINT: {
             DistributeEventConstKey.FROM_STATUS_LIST_KEY: [DistributeStatus.INIT, DistributeStatus.PRINTED,
@@ -537,6 +603,7 @@ class DistributeEventConst(object):
                 DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                 DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
                 DistributeStatus.SERVICE_STOP,
+                DistributeStatus.PACKAGED,
                 ],
             DistributeEventConstKey.TO_STATE_MAP_KEY: {
                     DistributeStatus.INIT: DistributeStatus.PRINTED,
@@ -569,7 +636,7 @@ class DistributeEventConst(object):
         SERVICEDESK_PICK_UP: {
             DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                 DistributeStatus.INIT, DistributeStatus.PRINTED,
-                DistributeStatus.CUSSERVICE_PICKED_UP, DistributeStatus.SERVICE_STORE],
+                DistributeStatus.CUSSERVICE_PICKED_UP, DistributeStatus.SERVICE_STORE, DistributeStatus.GUIDE_PICKED_UP],
             DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.SERVICE_DESK,
         },
         PACKAGE: {
@@ -582,19 +649,18 @@ class DistributeEventConst(object):
             DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.PACKAGED,
         },
         CANCEL_PACKAGE: {
-            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [DistributeStatus.PACKAGED],
+            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
+                DistributeStatus.PACKAGED
+                ],
             DistributeEventConstKey.TO_STATUS_KEY: None,
         },
         SERVICESTOP_CANCEL_PACKAGE: {
-            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [DistributeStatus.PACKAGED],
+            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
+                DistributeStatus.PACKAGED,
+                DistributeStatus.CUSSERVICE_PICKED_UP,
+                DistributeStatus.SERVICE_STORE],
             DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.SERVICE_STOP,
         },
-        # CUS_CONFIRM:{
-        #     DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
-        #         DistributeStatus.INIT, DistributeStatus.PRINTED, DistributeStatus.CUSSERVICE_PICKED_UP,
-        #         DistributeStatus.GUIDE_PICKED_UP,DistributeStatus.SERVICE_DESK],
-        #     DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.CUS_CONFIRM,
-        # },
         CUS_CONFIRM_AT_GUIDE: {
             DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                 DistributeStatus.INIT, DistributeStatus.PRINTED, DistributeStatus.CUSSERVICE_PICKED_UP,
@@ -643,33 +709,42 @@ class DistributeEventConst(object):
                 DistributeStatus.CUS_CONFIRM,
                 DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                 DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
+                DistributeStatus.CUS_CONFIRM_AT_VIRTUAL,
                 DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
                 DistributeStatus.PACKAGED, DistributeStatus.SERVICE_STOP,
                 DistributeStatus.OUTSTORAGE_PACKAGED,
                 DistributeStatus.HGVENDING_INIT, DistributeStatus.HGVENDING_WAIT_PICKUP, DistributeStatus.CUS_CONFIRM_AT_HGVENDING,
-                # DistributeStatus.HAITAO_INIT,
+                DistributeStatus.HAITAO_INIT,
                 DistributeStatus.DEPOSITPAY,
-                ],
+                DistributeStatus.TAILPAY_OVERTIME,
+                DistributeStatus.MAKEUP_POSTAGE_PAID,
+            ],
 
-            DistributeEventConstKey.TO_STATE_MAP_KEY: {DistributeStatus.INIT: DistributeStatus.SPLIT,
-                                                       DistributeStatus.PRINTED: DistributeStatus.SPLIT,
-                                                       DistributeStatus.CUSSERVICE_PICKED_UP: DistributeStatus.SPLIT,
-                                                       DistributeStatus.GUIDE_PICKED_UP: DistributeStatus.SPLIT,
-                                                       DistributeStatus.SERVICE_STORE: DistributeStatus.SPLIT,
-                                                       DistributeStatus.SERVICE_DESK: DistributeStatus.SPLIT,
-                                                       DistributeStatus.PACKAGED: None,
-                                                       DistributeStatus.CUS_CONFIRM: None,
-                                                       DistributeStatus.CUS_CONFIRM_AT_GUIDE: None,
-                                                       DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE: None,
-                                                       DistributeStatus.OUTSTORAGE_WAIT_PACKAGE: DistributeStatus.SPLIT,
-                                                       DistributeStatus.PACKAGED: None,
-                                                       DistributeStatus.OUTSTORAGE_PACKAGED: None,
-                                                       DistributeStatus.HGVENDING_INIT: None,
-                                                       DistributeStatus.HGVENDING_WAIT_PICKUP: None,
-                                                       DistributeStatus.CUS_CONFIRM_AT_HGVENDING: None,
-                                                       # DistributeStatus.HAITAO_INIT: None,
-                                                       DistributeStatus.DEPOSITPAY: None,
-                                                       },
+            DistributeEventConstKey.TO_STATE_MAP_KEY: {
+                DistributeStatus.INIT: DistributeStatus.SPLIT,
+                DistributeStatus.PRINTED: DistributeStatus.SPLIT,
+                DistributeStatus.CUSSERVICE_PICKED_UP: DistributeStatus.SPLIT,
+                DistributeStatus.GUIDE_PICKED_UP: DistributeStatus.SPLIT,
+                DistributeStatus.SERVICE_STORE: DistributeStatus.SPLIT,
+                DistributeStatus.SERVICE_DESK: DistributeStatus.SPLIT,
+                DistributeStatus.PACKAGED: None,
+                DistributeStatus.CUS_CONFIRM: None,
+                DistributeStatus.CUS_CONFIRM_AT_GUIDE: None,
+                DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE: None,
+                DistributeStatus.CUS_CONFIRM_AT_VIRTUAL: None,
+                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE: DistributeStatus.SPLIT,
+                DistributeStatus.PACKAGED: None,
+                DistributeStatus.OUTSTORAGE_PACKAGED: None,
+                DistributeStatus.HGVENDING_INIT: None,
+                DistributeStatus.HGVENDING_WAIT_PICKUP: None,
+                DistributeStatus.CUS_CONFIRM_AT_HGVENDING: None,
+                DistributeStatus.HAITAO_INIT: None,
+                DistributeStatus.DEPOSITPAY: None,
+                DistributeStatus.SERVICE_STOP: DistributeStatus.SPLIT,
+                # DistributeStatus.HAITAO_INIT: None,
+                DistributeStatus.TAILPAY_OVERTIME: None,
+                DistributeStatus.MAKEUP_POSTAGE_PAID: None,
+            },
         },
         CLOSE_AFTERSALE: {
             DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
@@ -677,34 +752,45 @@ class DistributeEventConst(object):
                 DistributeStatus.CUS_CONFIRM,
                 DistributeStatus.CUS_CONFIRM_AT_GUIDE,
                 DistributeStatus.CUS_CONFIRM_AT_CUSSERVICE,
+                DistributeStatus.CUS_CONFIRM_AT_VIRTUAL,
                 DistributeStatus.CUSSERVICE_HOLD, DistributeStatus.PACKAGED, DistributeStatus.OUTSTORAGE_PACKAGED,
                 DistributeStatus.OUTSTORAGE_WAIT_PACKAGE, DistributeStatus.OUTSTORAGE_RETURN,
                 DistributeStatus.INIT, DistributeStatus.PRINTED, DistributeStatus.CUSSERVICE_PICKED_UP,
                 DistributeStatus.GUIDE_PICKED_UP, DistributeStatus.SERVICE_DESK, DistributeStatus.SERVICE_STORE,
                 DistributeStatus.HGVENDING_INIT, DistributeStatus.HGVENDING_WAIT_PICKUP,
                 DistributeStatus.SERVICE_STOP,
+                DistributeStatus.DEPOSITPAY,
+                DistributeStatus.TAILPAY_OVERTIME,
                 ],
             DistributeEventConstKey.TO_STATUS_KEY: None,
         },
         COMPLETE_AFTERSALE: {
-            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [DistributeStatus.INIT,
-                                                        DistributeStatus.PRINTED,
-                                                        DistributeStatus.GUIDE_PICKED_UP,
-                                                        DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
-                                                        DistributeStatus.HGVENDING_INIT,
-                                                        DistributeStatus.HGVENDING_WAIT_PICKUP,
-                                                        DistributeStatus.HAITAO_INIT],
+            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
+                DistributeStatus.INIT,
+                DistributeStatus.PRINTED,
+                DistributeStatus.GUIDE_PICKED_UP,
+                DistributeStatus.OUTSTORAGE_WAIT_PACKAGE,
+                DistributeStatus.HGVENDING_INIT,
+                DistributeStatus.HGVENDING_WAIT_PICKUP,
+                DistributeStatus.CUS_CONFIRM_AT_VIRTUAL,
+                DistributeStatus.HAITAO_INIT,
+                DistributeStatus.DEPOSITPAY,
+                DistributeStatus.TAILPAY_OVERTIME,
+                ],
             DistributeEventConstKey.TO_STATE_MAP_KEY: {
                 DistributeStatus.INIT: DistributeStatus.NO_PICKUP_RETURN,
                 DistributeStatus.PRINTED: DistributeStatus.NO_PICKUP_RETURN,
                 DistributeStatus.GUIDE_PICKED_UP: DistributeStatus.NO_PICKUP_RETURN,
                 DistributeStatus.HGVENDING_INIT: DistributeStatus.HGVENDING_NO_PICKUP_RETURN,
                 DistributeStatus.HGVENDING_WAIT_PICKUP: DistributeStatus.HGVENDING_NO_PICKUP_RETURN,
-                DistributeStatus.HAITAO_INIT: DistributeStatus.HAITAO_RETURN
+                DistributeStatus.HAITAO_INIT: DistributeStatus.HAITAO_RETURN,
                 # DistributeStatus.OUTSTORAGE_WAIT_PACKAGE: DistributeStatus.OUTSTORAGE_RETURN
                 # DistributeStatus.CUSSERVICE_PICKED_UP:DistributeStatus.CUSSERVICE_HOLD,
                 # DistributeStatus.SERVICE_DESK:DistributeStatus.CUSSERVICE_HOLD,
                 # DistributeStatus.SERVICE_STORE:DistributeStatus.CUSSERVICE_HOLD,
+                DistributeStatus.DEPOSITPAY: DistributeStatus.DEPOSITPAY_RETURN,
+                DistributeStatus.TAILPAY_OVERTIME: DistributeStatus.DEPOSITPAY_RETURN,
+                DistributeStatus.CUS_CONFIRM_AT_VIRTUAL: None,
             }
         },
         CREATE_EXPRESS: {
@@ -800,7 +886,8 @@ class DistributeEventConst(object):
         },
         RETURN_DEPOSIT: {  # 分阶段支付--退定金
             DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
-                DistributeStatus.DEPOSITPAY
+                DistributeStatus.DEPOSITPAY,
+                DistributeStatus.TAILPAY_OVERTIME,
             ],
             DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.DEPOSITPAY_RETURN
         },
@@ -818,15 +905,91 @@ class DistributeEventConst(object):
         },
         HAITAO_PACKAGE: {
             DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
-                                DistributeStatus.HAITAO_INIT],
+                                DistributeStatus.HAITAO_BAISHI],
             DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.HAITAO_PACKAGED
         },
+        HAITAO_PUSHDATA_2_BAISHI: {
+            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
+                DistributeStatus.HAITAO_INIT],
+            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.HAITAO_BAISHI
+        },
+        HAITAO_CANCEL_FROM_BAISHI: {
+            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
+                DistributeStatus.HAITAO_BAISHI],
+            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.HAITAO_INIT
+        },
+
         PUSH_GUIDE_ASSIGN: {
             DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
                                 DistributeStatus.PRINTED,
                                 DistributeStatus.INIT],
             DistributeEventConstKey.TO_STATUS_KEY: None
-        }
+        },
+
+        # 2022.08 特殊售后单需求
+        AFSPLUS_OPEN: {
+            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
+                DistributeStatus.INIT, DistributeStatus.PRINTED,
+                                DistributeStatus.CUSSERVICE_PICKED_UP,
+                                DistributeStatus.GUIDE_PICKED_UP,
+                                DistributeStatus.SERVICE_DESK,
+                                DistributeStatus.SERVICE_STORE,
+                                DistributeStatus.SERVICE_STOP,
+            ],
+            DistributeEventConstKey.TO_STATUS_KEY: None
+        },
+        AFSPLUS_CLOSE: {
+            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
+                DistributeStatus.INIT, DistributeStatus.PRINTED,
+                                DistributeStatus.CUSSERVICE_PICKED_UP,
+                                DistributeStatus.GUIDE_PICKED_UP,
+                                DistributeStatus.SERVICE_DESK,
+                                DistributeStatus.SERVICE_STORE,
+                                DistributeStatus.SERVICE_STOP,
+            ],
+            DistributeEventConstKey.TO_STATUS_KEY: None
+        },
+        AFSPLUS_COMPLETE: {
+            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
+                DistributeStatus.INIT, DistributeStatus.PRINTED,
+                                DistributeStatus.CUSSERVICE_PICKED_UP,
+                                DistributeStatus.GUIDE_PICKED_UP,
+                                DistributeStatus.SERVICE_DESK,
+                                DistributeStatus.SERVICE_STORE,
+                                DistributeStatus.SERVICE_STOP,
+            ],
+            DistributeEventConstKey.TO_STATUS_KEY: None
+        },
+        AFSPLUS_S2E: {
+            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
+                DistributeStatus.INIT, DistributeStatus.PRINTED,
+                                DistributeStatus.CUSSERVICE_PICKED_UP,
+                                DistributeStatus.GUIDE_PICKED_UP,
+                                DistributeStatus.SERVICE_DESK,
+                                DistributeStatus.SERVICE_STORE,
+                                DistributeStatus.SERVICE_STOP,
+            ],
+            DistributeEventConstKey.TO_STATUS_KEY: None
+        },
+        AFSPLUS_E2S: {
+            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
+                DistributeStatus.INIT, DistributeStatus.PRINTED,
+                                DistributeStatus.CUSSERVICE_PICKED_UP,
+                                DistributeStatus.GUIDE_PICKED_UP,
+                                DistributeStatus.SERVICE_DESK,
+                                DistributeStatus.SERVICE_STORE,
+                                DistributeStatus.SERVICE_STOP,
+            ],
+            DistributeEventConstKey.TO_STATUS_KEY: None
+        },
+
+        # 2022.08 虚拟商品确收事件
+        CUS_CONFIRM_AT_VIRTUAL: {
+            DistributeEventConstKey.FROM_STATUS_LIST_KEY: [
+                DistributeStatus.VIRTUAL_INIT],
+            DistributeEventConstKey.TO_STATUS_KEY: DistributeStatus.CUS_CONFIRM_AT_VIRTUAL,
+        },
+
     }
 
     EVENT_TO_BUTTONNAME_MAP = {
@@ -861,10 +1024,12 @@ class DistributeEventConst(object):
         CREATE_EXPRESS: "快递下单",
         CANCEL_EXPRESS: "取消快递下单",
         MODIFY_SHIPPING_NUMBER: "修改运单号",
-        MERGE_EXPRESS: "合并发货",
+        UPDATE_SHIPPING_METHOD_FOR_MERGE: "合单-修改发货方式",
+        MERGE_EXPRESS: "合单-修改运单号-辅单",
+        MERGE_EXPRESS_MAIN: "合单-修改运单号-主单",
         CUSSERVICE_RETURN: "客服还货",
         OUTSTORAGE_RETURN: "外仓还货",
-        Add_SHIPPING_ADDRESS: "新增发货地址",
+        ADD_SHIPPING_ADDRESS: "新增发货地址",
         UPDATE_SHIPPING_METHOD: "修改发货方式",
         ADD_GUIDE_NOTE: "导购备注",
         BLOCK_SUCC: "拦截成功",
@@ -895,9 +1060,20 @@ class DistributeEventConst(object):
         HAITAO_PACKAGE: "海淘已发货",
         # 海淘推数据给百世
         HAITAO_PUSHDATA_2_BAISHI: "海淘推数据给百世",
+        HAITAO_CANCEL_FROM_BAISHI: "海淘取消推百世",
         UPDATE_DELIVER_TIME: "更新发货时间",
         # 合单催配货消息功能
-        PUSH_GUIDE_ASSIGN: "合单催配送提醒"
+        PUSH_GUIDE_ASSIGN: "合单催配送提醒",
+
+        # 特殊售后单
+        AFSPLUS_OPEN: "申请特殊售后",
+        AFSPLUS_CLOSE: "取消特殊售后",
+        AFSPLUS_COMPLETE: "完成特殊售后",
+        AFSPLUS_S2E: "自提转快递",  # 自提转快递
+        AFSPLUS_E2S: "快递转自提",  # 快递转自提
+
+        # 虚拟商品
+        CUS_CONFIRM_AT_VIRTUAL: "虚拟商品确收"
     }
 
 
@@ -917,7 +1093,6 @@ def get_available_event_list(distribute_status):
         DistributeEventConst.ADD_GUIDE_NOTE,
         DistributeEventConst.CUSSERVICE_RETURN,
         DistributeEventConst.OUTSTORAGE_RETURN,
-        DistributeEventConst.ERROR_MESSAGE,
         DistributeEventConst.EXCHANGE_OPEN,
         DistributeEventConst.EXCHANGE_CANCEL,
         DistributeEventConst.EXCHANGE_DONE,
@@ -935,7 +1110,7 @@ def get_available_event_list(distribute_status):
             continue
         from_status_list = event_dict.get(DistributeEventConstKey.FROM_STATUS_LIST_KEY)
         if distribute_status in from_status_list and event_name not in special_event_list:
-            event_str = event_name + "|" + DistributeEventConst.EVENT_TO_BUTTONNAME_MAP.get(event_name)
+            event_str = event_name + "|" + DistributeEventConst.EVENT_TO_BUTTONNAME_MAP.get(event_name, event_name)
             available_event_list.append(event_str)
     return available_event_list
 
@@ -987,3 +1162,108 @@ class StepPayDeliverTimeType:
 
     RELATIVE_TIME = "pay_relative_time"  # 相对发货时间  尾款支付成功后多久可以发货
     FIXED_TIME = "fixed_time"  # 绝对时间
+
+
+# 欧莱雅集团可同步专柜列表
+OLAY_SHOP_DICT_CONST = {
+    "108030": "01",
+    "108048": "45",
+    "108063": "15"
+}
+
+class AddressGroupStatus(object):
+    """
+    可合-已齐、可合-未齐、无可合、已合单、单单、其他
+    """
+    # 可合-已齐
+    MERGABLE_READY = "mergable_ready"
+    # 可合-未齐
+    MERGABLE_WAITING = "mergable_waiting"
+    # 已合单
+    MERGED = "merged"
+    # 在库-已合
+    MERGED_IN_STORE = "merged_in_store"
+    # 无可合
+    UNMERGABLE = "unmergable"
+    # 单单
+    SINGLE = 'single'
+    # 其他
+    OTHER = 'other'
+
+    CHOICES = (
+        (MERGABLE_READY, '可合-已齐'),
+        (MERGABLE_WAITING, '可合-未齐'),
+        (MERGED, "已合单"),
+        (MERGED_IN_STORE, "在库-已合"),
+        (UNMERGABLE, "无可合"),
+        (SINGLE, "单单"),
+        (OTHER, "其他"),
+    )
+    # 存在未终结发货单的分组
+    UNTERMINATE_STATUS_LIST = [MERGABLE_READY,MERGABLE_WAITING, MERGED, UNMERGABLE, SINGLE]
+
+"""
+发货单运送变更事件 枚举
+合单、更新运单
+TODO 修改地址、修改运输方式
+"""
+@unique
+class DistributeShippingChangeEventEnum(Enum):
+
+    merge_express_order = "merge_express_order"
+    create_express_order = "create_express_order"
+    update_shipping_number = "update_shipping_number"
+
+
+    @classmethod
+    def choices(cls):
+        return (
+            (cls.merge_express_order, "合单"),
+            (cls.update_shipping_number, "更新运单"),
+            (cls.create_express_order, "创建运单"),
+        )
+
+"""
+拣货任务创建途径
+"""
+class MergePickPickTaskSource(object):
+    # 揽月领任务
+    LANYUE_ASSIGN_PICKTASK = 'lanyue_assign_picktask'
+    # PDA扫发货单创建拣货任务
+    PDA_SCAN_CREATE = 'pda_scan_create'
+
+    CHOICES = (
+        (LANYUE_ASSIGN_PICKTASK, '揽月领任务'),
+        (PDA_SCAN_CREATE, 'PDA扫码创建任务'),
+    )
+
+# 合单拣货任务状态
+class MergePickTaskStatus(object):
+    # 已领取
+    CLAIMED = "claimed"
+    # 已完成
+    COMPLETED = 'completed'
+    # 已关闭
+    CLOSED = 'closed'
+
+    CHOICES = (
+        (CLAIMED, '已领取'),
+        (COMPLETED, '已完成'),
+        (CLOSED, "已关闭"),
+    )
+
+    NAME_MAP = dict(CHOICES)
+
+# 合单拣货任务覆盖 address group
+class MergePickTaskCoverAddressGroup(object):
+    # 部分
+    PART = "part"
+    # 全部
+    ALL = 'all'
+
+    CHOICES = (
+        (PART, '部分'),
+        (ALL, '全部'),
+    )
+
+    NAME_MAP = dict(CHOICES)
